from emod_api import schema_to_class as s2c
import emod_api.interventions.common as common
import emod_api.interventions.utils as utils
from emodpy_tbhiv.interventions import purge_campaign_event

co_event = "BcgDistributed"

def BCG(
        camp,
        trigger_treatment_list,
        initial_efficacy=1.0,
        vaccine_take=1,
        age_take_decay=1.0,
        box_duration_yrs=1,
        immune_decay_yrs=0, 
        start_day=1,
        duration=-1,
        property_restrictions_list=[],
        nodeIDs=[],
        black_period=0,
        black_trigger='Blackout',
        event_name='Vaccine Health Seeking'):
    """
    Create and return triggered campaign event that issues an BCG vaccine intervention.

    Args:

        camp: Centralized campaign module instance.
        trigger_treatment_list: List of 1 or more triggers (or events or signals) which are listened to and trigger the distribution of the intervention. There is no default. 
        initial_efficacy: Initial efficacy of the vaccine. Defaults to 1.0.
        vaccine_take: Fraction of the population receiving the vaccine for whom it is efficacious. Defaults to 1.0.
        age_take_decay: ... Defaults to 1.0.
        box_duration_yrs: Period of time over which the initial efficacy persists before decay. Defaults to 1 year.
        immune_decay_yrs: Period of time over which the efficacy decays to 0. Defaults to 0.

        start_day: The timestep when this campaign event takes effect. Defaults to 1.
        duration: How long the campaign event remains in effect. Defaults to forever.
        property_restrictions_list: Optiional list of Individual Properties to limit the intervention to.
        nodeIDs: Optiona list of node ids to target. Defaults to all.

        black_period: Undocumented.
        black_trigger: Undocumented.
        event_name: Undocumented.

    Returns:

        New campaign event that can be added to the campaign.

    """
    if box_duration_yrs > 150:
        raise ValueError( f"It seems that your value of box_duration_yrs ({box_duration_yrs}) might be in units of days instead of years." )
    if immune_decay_yrs > 150:
        raise ValueError( f"It seems that your value of immune_duration_yrs ({immune_duration_yrs}) might be in units of days instead of years." )

    # First, get the objects
    schema_path = camp.schema_path

    act_intervention = s2c.get_class_with_defaults( "BCGVaccine", schema_path )
    waning = s2c.get_class_with_defaults( "WaningEffectBoxExponential", schema_path )
    bcast_intervention = s2c.get_class_with_defaults( "BroadcastEvent", schema_path )

    # Second, hook them up
    act_intervention.Waning_Config = waning
    bcast_intervention.Broadcast_Event = camp.get_send_trigger( co_event )

    # Third, do the actual settings
    act_intervention.Vaccine_Take = vaccine_take
    act_intervention.Vaccine_Take_Age_Decay_Rate = age_take_decay
    waning.Initial_Effect = initial_efficacy
    waning.Box_Duration = box_duration_yrs*365
    waning.Decay_Time_Constant = immune_decay_yrs*365

    event = common.TriggeredCampaignEvent( camp, start_day,event_name, trigger_treatment_list, [ act_intervention, bcast_intervention ], Node_Ids=nodeIDs, Duration=duration, Property_Restrictions=property_restrictions_list )
    purge_campaign_event( event )
    return event

def new_intervention_as_file( camp, filename=None ):
    camp.add( BCG( camp, trigger_treatment_list=[ "Births" ] ) )
    if filename is None:
        filename = "bcg.json"
    camp.save( filename )
    return filename

